#! /usr/bin/env python
# -*- coding: utf8 -*-
"""Binary file translator for LeCROY osciloscope"""

import struct
import array
    
# data endian
# '>' big endian  (in case of GPIB)
# '<' little endian
default_endian_prefix = '>'

fmtBYTE='c'
fmtWORD='h'
fmtLONG='l'
fmtFLOAT='f'
fmtDOUBLE='d'
fmtSTRING='16s'
fmtTEXT='160s'
fmtUNIT_DEFINITION='48s'

time_stamp_struct_format = [
    ('seconds', fmtDOUBLE),
    ('minutes', fmtBYTE),
    ('hours', fmtBYTE),
    ('days', fmtBYTE),
    ('manths', fmtBYTE),
    ('year', fmtWORD),
    ('unused', fmtWORD)
]

fmtTIMESTAMP=''.join([fmt for (name, fmt) in time_stamp_struct_format])

wavedesc_struct_format = [
    ('DESCRIPTOR_NAME', fmtSTRING),
    ('TEMPLATE_NAME', fmtSTRING),
    ('COMM_TYPE', fmtWORD),
    ('COMM_ORDER', fmtWORD),
    ('WAVE_DESCRIPTOR', fmtLONG),
    ('USER_TEXT', fmtLONG),
    ('RES_DESC1', fmtLONG),
    ('TRIGTIME_ARRAY', fmtLONG),
    ('RIS_TIME_ARRAY', fmtLONG),
    ('RES_ARRAY1', fmtLONG),
    ('WAVE_ARRAY_1', fmtLONG),
    ('WAVE_ARRAY_2', fmtLONG),
    ('RES_ARRAY2', fmtLONG),
    ('RES_ARRAY3', fmtLONG),
    ('INSTRUMENT_NAME', fmtSTRING),
    ('INSTRUMENT_NUMBER', fmtLONG),
    ('TRACE_LABEL', fmtSTRING),
    ('RESERVED1', fmtWORD),
    ('RESERVED2', fmtWORD),
    ('WAVE_ARRAY_COUNT', fmtLONG),
    ('PNTS_PER_SCREEN', fmtLONG),
    ('FIRST_VALID_PNT', fmtLONG),
    ('LAST_VALID_PNT', fmtLONG),
    ('FIRST_POINT', fmtLONG),
    ('SPARSING_FACTOR', fmtLONG),
    ('SEGMENT_INDEX', fmtLONG),
    ('SUBARRAY_COUNT', fmtLONG),
    ('SWEEPS_PER_ACQ', fmtLONG),
    ('POINTS_PER_PAIR', fmtWORD),
    ('PAIR_OFFSET', fmtWORD),
    ('VERTICAL_GAIN', fmtFLOAT),
    ('VERTICAL_OFFSET', fmtFLOAT),
    ('MAX_VALUE', fmtFLOAT),
    ('MIN_VALUE', fmtFLOAT),
    ('NOMINAL_BITS', fmtWORD),
    ('NOM_SUBARRAY_COUNT', fmtWORD),
    ('HORIZ_INTERVAL', fmtFLOAT),
    ('HORIZ_OFFSET', fmtDOUBLE),
    ('PIXEL_OFFSET', fmtDOUBLE),
    ('VERTUNIT', fmtUNIT_DEFINITION),
    ('HORUNIT', fmtUNIT_DEFINITION),
    ('RESERVED3', fmtWORD),
    ('RESERVED4', fmtWORD),
    ('TRIGGER_TIME', fmtTIMESTAMP),
    ('ACQ_DURATION', fmtFLOAT),
    ('RECODE_TYPE', fmtWORD),
    ('PROCESSING_DONE', fmtWORD),
    ('RESERVED5', fmtWORD),
    ('RIS_SWEEPS', fmtWORD),
    ('TIMEBASE', fmtWORD),
    ('VERT_COUPLING', fmtWORD),
    ('PROBE_ATT', fmtFLOAT),
    ('FIXED_VERT_GAIN', fmtWORD),
    ('BANDWIDTH_LIMIT', fmtWORD),
    ('VERTICAL_VERNIER', fmtFLOAT),
    ('ACQ_VERT_OFFSET', fmtFLOAT),
    ('WAVE_SOURCE', fmtWORD) ]

fmtWAVEDESC=''.join([fmt for (name,fmt) in wavedesc_struct_format])

class TimeStamp(object):
    format_list = time_stamp_struct_format

    def __init__(self, endian=default_endian_prefix):
        self.endian_prefix = endian
        for (name, fmt) in self.format_list:
            if fmt==fmtBYTE or fmt==fmtWORD or fmt==fmtLONG :
                self.__dict__[name] = 0
            elif fmt==fmtFLOAT or fmt==fmtDOUBLE :
                self.__dict__[name] = 0.0
            else :
                raise 'unknown type %s' % fmt

    def fromstring(self, fbuf):
        start = stop = 0
        for (name, fmt) in self.format_list:
            prefmt = self.endian_prefix + fmt
            if fmt==fmtBYTE or fmt==fmtWORD or fmt==fmtLONG or \
               fmt==fmtFLOAT or fmt==fmtDOUBLE :
                stop = start + struct.calcsize(prefmt)
                (self.__dict__[name],) = struct.unpack(prefmt, fbuf[start:stop])
            else :
                raise 'unknown type %s' % fmt
            start = stop

    def format_string(self):
        return "%4d/%02d/%02d %02d:%02d:%6.3f" % \
               (self.year, ord(self.manths), ord(self.days),
                 ord(self.hours), ord(self.minutes), self.seconds)

class Wavedesc(object):
    format_list = wavedesc_struct_format

    def __init__(self, endian=default_endian_prefix):
        self.endian_prefix = endian
        for (name, fmt) in self.format_list:
            if fmt==fmtBYTE or fmt==fmtWORD or fmt==fmtLONG :
                self.__dict__[name] = 0
            elif fmt==fmtFLOAT or fmt==fmtDOUBLE :
                self.__dict__[name] = 0.0
            elif fmt == fmtSTRING or fmt==fmtTEXT or fmt == fmtUNIT_DEFINITION :
                self.__dict__[name] = ''
            elif fmt == fmtTIMESTAMP :
                self.__dict__[name] = TimeStamp(self.endian_prefix)
            else :
                raise 'unknown type %s' % fmt

    def fromstring(self, fbuf):
        start = stop = 0
        for (name, fmt) in self.format_list:
            prefmt = self.endian_prefix + fmt
            if fmt==fmtBYTE or fmt==fmtWORD or fmt==fmtLONG or \
               fmt==fmtFLOAT or fmt==fmtDOUBLE :
                stop = start + struct.calcsize(prefmt)
                (self.__dict__[name],) = struct.unpack(prefmt, fbuf[start:stop])
            elif fmt==fmtSTRING or fmt==fmtTEXT or fmt==fmtUNIT_DEFINITION:
                stop = start + struct.calcsize(prefmt)
                (self.__dict__[name],) = struct.unpack(prefmt, fbuf[start:stop])
                self.__dict__[name] = self.__dict__[name].strip('\0')
            elif fmt == fmtTIMESTAMP :
                stop = start + struct.calcsize(prefmt)
                self.__dict__[name].fromstring(fbuf[start:stop])
            else :
                raise 'unknown type %s' % fmt
            start = stop

    def format_string(self):
        string=''
        for (name, fmt) in self.format_list:
            if fmt==fmtBYTE or fmt==fmtWORD or fmt==fmtLONG or \
               fmt==fmtFLOAT or fmt==fmtDOUBLE or \
               fmt==fmtSTRING or fmt==fmtTEXT or fmt==fmtUNIT_DEFINITION:
                string = string + name + ':' + str(self.__dict__[name]) + '\n'
            elif fmt == fmtTIMESTAMP :
                string = string + name + ':' + self.__dict__[name].format_string() + '\n'
            else :
                raise 'unknown type %s' % fmt

        return string

#
def translate(fname, endian='<'):
    f = open(fname, 'rb')
    fbuf = f.read()
    pbeg=fbuf.find('WAVEDESC')
    if pbeg==-1:
        raise "cannot find header kyeword 'WAVEDESC'"
    
    # load header
    w = Wavedesc(endian)
    w.fromstring(fbuf[pbeg:])

    # load data array
    abeg = pbeg + w.WAVE_DESCRIPTOR
    aend = abeg + w.WAVE_ARRAY_1

    arrayform=''
    if w.COMM_TYPE == 0 :
        arrayform = 'b'
        aend = abeg + w.WAVE_ARRAY_1
    else:
        arrayform = 'h'
        aend = abeg + w.WAVE_ARRAY_1 * 2
    ary = array.array(arrayform, fbuf[abeg:aend])

    # returns pair of header, and data array
    return (w, ary)


if __name__ == '__main__' :
    import sys
    import getopt

    endian_prefix = default_endian_prefix

    opts, args = getopt.getopt(sys.argv[1:], '', ['big-endian', 'little-endian'])
    for o, a in opts:
        if o == '--big-endian':
            endian_prefix = '>'
        elif o == '--little-endian':
            endian_prefix = '<'
        else:
            print >> sys.stderr, 'unknown option', o
            sys.exit(1)

    if len(args) < 1:
        print "few parameters"
        print "usage:"
        print "  lecroy [options] <input> [output]"
        print "   input: file name of wave template"
        print "  output: file name of text data "
        print "          (if not secified, output to standart output)"
        print "options:"
        print "  --big-endian: treat as big-endian datatype (default)"
        print "  --little-endian: treat as little-endian datatype"
        sys.exit(3)
        
    try :
        (w, ary) = translate(args[0], endian_prefix)
    except:
        print 'error in reading file %s' % args[0]
        sys.exit(3)

    of = sys.stdout
    if len(args) > 1:
        of = open(argv[1], "wt")

    of.write(w.format_string())
    of.write("ATA_ARRAY_1:\n")
    
    for i, a in enumerate(ary):
        of.write("%10d %g %10d %10f\n" % \
                  (i ,
                   i * w.HORIZ_INTERVAL,
                   a,
                   a * w.VERTICAL_GAIN - w.VERTICAL_OFFSET))

    of.close()
