#! /usr/bin/env python
# -*- coding: utf8 -*-
"""Binary file translator for Stanford SR430"""

import struct
import array

class Sr430TraceHeader(object):
    """Header definition for SR430"""
    structure_list = [
        ( 'header_text', '12s' ),      # 0-11
        ( 'bin_width', 'h' ),          # 12
        ( 'reserved1', 'x'* 2 ),       # 14-15
        ( 'bins_per_record', 'h' ),    # 16
        ( 'reserved2', 'x'*(35-18+1)), # 18-35
        ( 'minimum_data_value', 'f'),  # 36-39
        ( 'range_of_data', 'f' ),      # 40-43
        ( 'records_accumulatd', 'i' )  # 44-47
    ]

    fmt_for_struct = ''.join([fmt for (name, fmt) in structure_list])
    header_length = struct.calcsize(fmt_for_struct)

    bin_width_tbl = [ 5e-9, 40e-9, 80e-9, 160e-9, 320e-9, 640e-9,
            1.28e-6, 2.56e-6, 5.12e-6, 10.24e-6, 20.48e-6, 40.96e-6, 
            81.92e-6, 163.48e-6, 327.68e-6, 655.36e-6,
            1.3107e-3, 2.6214e-3, 5.2429e-3, 10.486e-3]


    def __init__(self): 
        self.header_text = 'SR430_TRACE\r'
        self.bin_width = 0
        self.bins_per_record = 1
        self.minimum_data_value = 0.0
        self.range_of_data = 0.0
        self.records_accumulated = 0


    def bin_time_width(self):
        return Sr430TraceHeader.bin_width_tbl[self.bin_width]


    def num_of_bins(self):
        return 1024 * self.bins_per_record


    def from_string(self, fbuf):
        (self.header_text,
         self.bin_width,
         self.bins_per_record,
         self.minimum_data_value,
         self.range_of_data,
         self.records_accumulated) = struct.unpack(Sr430TraceHeader.fmt_for_struct, fbuf[:Sr430TraceHeader.header_length])
    

    def to_string(self):
        struct.pack(Sr430TraceHeader.fmt_for_struct,
                    (self.header_text, self.bin_width, self.bins_per_record,
                        self.minimum_data_value, self.range_of_data, self.records_accumulated))


    def __str__(self):
        string = ''
        string += 'bin_width:           %d (%e sec)\n' % (self.bin_width, self.bin_time_width())
        string += 'bins_per_record:     %d (%d)\n' % (self.bins_per_record, self.num_of_bins())
        string += 'minimum_data_value:  %e\n' % self.minimum_data_value
        string += 'range_of_data:       %e\n' % self.range_of_data
        string += 'records_accumulated: %ld' % self.records_accumulated
        return string



def translate(fname):
    """read a file named 'fname', and returns a tuple of a header and bindata array"""
    f = open(fname, 'rb')
    fbuf = f.read()
    
    # create header structure
    h = Sr430TraceHeader()
    h.from_string(fbuf)

    # read array of bindata
    abeg = Sr430TraceHeader.header_length
    aend = abeg + h.num_of_bins() * struct.calcsize('h')
    ary=array.array('h', fbuf[abeg:aend] )

    return (h, ary)


# when this is called as a main proguram
if __name__ == '__main__' :
    import sys

    if len(sys.argv) < 2 :
        print "few parameters"
        print "%s <input> [output]" % sys.argv[0]
        print " input: file name of SR430 trace file (binary)"
        print "output: file name of text data "
        print "        ( if not secified, output to standart output)"
        sys.exit(3)
        
    #open file and parse it
    (h, ary) = translate(sys.argv[1])

    #output to file
    of = sys.stdout
    if len(sys.argv) > 2:
        of = open( sys.argv[2], "wt" )

    print >> of, 'filename:            %s' % sys.argv[1]
    print >> of, h
    print >> of, '------------------------------------'
    print >> of, 'bin\ttime\tcount'

    for i, a in enumerate(ary):
        print >> of, "%ld\t%e\t%d" % (i, i * h.bin_time_width(), a)

    of.close()
